/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.extractors;

import java.util.Iterator;
import java.util.Set;

import it.eng.configurazione.bean.ExtractionProcess;
import it.eng.configurazione.bean.Operation;

/**
 * This coordinator should be able to manage more operations
 * for the same extraction process
 * Now only one operation is managed
 */

public class DefaultOperationCoordinator implements
		OperationCoordinatorInterface {

	private ExtractionProcess process=null;
	
	private ExtratorInterface extractor=null;
	
	public DefaultOperationCoordinator(){
		
	}
	
	public void init(ExtractionProcess process){
		this.process=process;
		
		// get the list of operations defided for the process
		Set<Operation> operations=process.getOperation();
		Iterator<Operation> iter=operations.iterator();

		// get the fist one ...
		Operation operation=iter.next();
		String extractorClass=operation.getDataSource().getSource().getClassforname();
		
		try {
			extractor=(ExtratorInterface)Class.forName(extractorClass).newInstance();
			extractor.init(process);
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		
		
	}
	
	public boolean hasNext() {

		return extractor.hasNext();
	}


	public GenericItemInterface nextItem() {

		return extractor.nextItem();
	}

}
