/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.extractors.svn;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringBufferInputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DecimalFormat;
import java.util.Vector;

import javancss.Javancss;
import javancss.JavancssConstants;

public class MetricsJava {

	public static Object execMethod(Object classMethod, String nameMethod) {
		Object retVal = null;
		try {
			Method m = classMethod.getClass().getMethod(nameMethod, new Class[0]);
			retVal = (Object) m.invoke(classMethod, new Object[0]);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return retVal;
	}
	
	public Object execMethod(Object classMethod, String nameMethod, Object args[]) {

		Object retVal = null;
		Class c = classMethod.getClass();
		Class parametriFormali[] = null;
		int argNum = 0;

		if (args!=null) {
			argNum = args.length;
			parametriFormali = new Class[argNum];
			for (int i=0; i<argNum-1; i++) {
				parametriFormali[i] = args[i].getClass();
			}
			parametriFormali[argNum-1] = args[argNum-1].getClass();
		}

		Method m = null;
		try { 
			// trova solo metodi senza tipi primitivi
			m = c.getMethod(nameMethod, parametriFormali);
		} catch (NoSuchMethodException e){
			System.err.println("Metodo inesistente");
		}

		try {
			retVal = m.invoke(classMethod, args);
		} catch(InvocationTargetException itE){
			itE.printStackTrace();
		} catch(IllegalAccessException iaE){
			iaE.printStackTrace();
		}

		return retVal;

	}

	/**
	 * Returns number of line code. 
	 * @param  baos ByteArrayOutputStream stream da analizzare
	 * @return      LOC (numero di linee di codice)
	 */
	public int getLOC(ByteArrayOutputStream baos) throws IOException {
		int retVal = 0;
	
		Javancss pJavancss = new Javancss(new StringBufferInputStream(baos.toString()));
		retVal = pJavancss.getLOC();// numero lenee codice

		return retVal;
	}
	
	/**
	 * Returns percentuale di linee commentate rispetto a quelle di codice. 
	 * I commenti comprendono commenti di righe singole e multiple oltre alle righe vuote
	 * @param  baos ByteArrayOutputStream stream da analizzare
	 * @return      CLOC (percentuale di linee commentate)
	 */
	public String getCLOC(ByteArrayOutputStream baos) throws IOException {
		double retVal = 0;

		Javancss pJavancss = new Javancss(new StringBufferInputStream(baos.toString()));
		
		int numeroRigheCodice = pJavancss.getLOC();// numero righe codice
		int numeroRigheNoComment = pJavancss.getNcss(); // numero righe senza commento

		int [] values = {numeroRigheCodice, numeroRigheNoComment};
		retVal = (double)numeroRigheNoComment * 100 / (double)numeroRigheCodice;
		
		DecimalFormat df1 = new DecimalFormat("####.0");
		return df1.format(retVal);
	}
	
	/**
	 * Returns Cyclomatic Complexity 
	 * @param  baos ByteArrayOutputStream stream da analizzare
	 * @return      CC (complessit ciclomatica)
	 */
	public int getCC(ByteArrayOutputStream baos) throws IOException {
		int retVal = 0;
	
		Javancss pJavancss = new Javancss(new StringBufferInputStream(baos.toString()));

	    for (int index=0; index<pJavancss.getFunctionMetrics().size(); index++){
	    	retVal += getCCN(pJavancss.getFunctionMetrics(),index);
	    }

		return retVal;
	}
    
    private static int getCCN(Vector vFunctions, int methodIndex) {
		return ((Integer)((Vector)vFunctions.elementAt(methodIndex)).elementAt(JavancssConstants.FCT_CCN)).intValue();
	}

    /**
	 * Returns Numero di inner class.
	 * @param  baos ByteArrayOutputStream stream da analizzare
	 * @return      NC (Numero di inner class)
	 */
	public int getNC(ByteArrayOutputStream baos) throws IOException {
		int retVal = 0;
	
		Javancss pJavancss = new Javancss(new StringBufferInputStream(baos.toString()));

		for (int index=0; index<pJavancss.getObjectMetrics().size(); index++){
        	Vector vObject = pJavancss.getObjectMetrics();
        	retVal += (Integer)((Vector)vObject.elementAt(index)).elementAt(JavancssConstants.OBJ_CLSSS);
		}

		return retVal;
	}

    /**
	 * Returns Numero di package.
	 * @param  baos ByteArrayOutputStream stream da analizzare
	 * @return      NP (Numero di package)
	 */
	public int getNP(ByteArrayOutputStream baos) throws IOException {
		int retVal = 0;
	
		Javancss pJavancss = new Javancss(new StringBufferInputStream(baos.toString()));
		retVal = pJavancss.getPackage().length;

		return retVal;
	}
    
    /**
	 * Returns nome del package.
	 * @param  baos ByteArrayOutputStream stream da analizzare
	 * @return      PACKAGE_NAME (Nome del package)
	 */
	public String getPACKAGE_NAME(ByteArrayOutputStream baos) throws IOException {
		String retVal = "";
	
		Javancss pJavancss = new Javancss(new StringBufferInputStream(baos.toString()));

		retVal = pJavancss.getPackageMetrics().firstElement().toString();
		
		return retVal;
	}

	
}
