package it.eng.configurazione.bean;

import it.eng.cipher.DefaultCipher;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.CollectionOfElements;

@Entity
@Table(name="E_DATA_SOURCE")

public class DataSource {

	private int id;
	private String name=null;
	private String description=null;
	private SourceType source;
	private Set<ConnetionParameter> parameters=null;
	
	@Column( nullable = true, length=200)
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	@Column( nullable = false, length=50 ,unique=true)
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	@ManyToOne( cascade = {CascadeType.PERSIST, CascadeType.MERGE} )
	@JoinColumn(name="type_id")	
	public SourceType getSource() {
		return source;
	}
	public void setSource(SourceType source) {
		this.source = source;
	}
	@CollectionOfElements
	public Set<ConnetionParameter> getParameters() {
		return parameters;
	}
	public void setParameters(Set connetionParameters) {
		this.parameters = connetionParameters;
	}
	public void addParameter(ConnetionParameter p){
		if (this.parameters==null){
			this.parameters=new HashSet<ConnetionParameter>();
		}
		this.parameters.add(p);
	}

	/**
	 * ritorna il valore di un parametro
	 * @param parName
	 * @return
	 */
	public String readConnetionParameterValue(String parName){
		Iterator<ConnetionParameter> iter=this.parameters.iterator();
		while (iter.hasNext()){
			ConnetionParameter tmp=iter.next();
			if (tmp.getName().equals(parName)) return tmp.getValue();
		}
		return null;
	}
	
	/**
	 * ritorna il valore di un parametro decifrandolo
	 * @param parName
	 * @return
	 */
	public String readConnetionHideParameterValue(String parName){
		DefaultCipher defaultCipher = new DefaultCipher();
		Iterator<ConnetionParameter> iter=this.parameters.iterator();
		while (iter.hasNext()){
			ConnetionParameter tmp=iter.next();
			if (tmp.getName().equals(parName)) return defaultCipher.decrypt(tmp.getValue());
		}
		return null;
	}
	
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + id;
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final DataSource other = (DataSource) obj;
		if (id != other.id)
			return false;
		return true;
	}
}
